@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -482,6 +482,24 @@ public class LafWidgetUtilities {
 				.get(LafWidget.TEXT_EDIT_CONTEXT_MENU)));
 	}
 
+	/**
+	 * Checks whether the specified scroll pane supports auto scroll.
+	 * 
+	 * @param scrollPane
+	 *            Scroll pane component.
+	 * @return <code>true</code> if the specified scroll pane supports auto
+	 *         scroll, <code>false</code> otherwise.
+	 */
+	public static boolean hasAutoScroll(JScrollPane scrollPane) {
+		Object compProperty = scrollPane
+				.getClientProperty(LafWidget.AUTO_SCROLL);
+		if (Boolean.TRUE.equals(compProperty))
+			return true;
+		if (Boolean.FALSE.equals(compProperty))
+			return false;
+		return (Boolean.TRUE.equals(UIManager.get(LafWidget.AUTO_SCROLL)));
+	}
+
 	/**
 	 * Checks whether the specified tree component has automatic drag and drop
 	 * support.
