@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -378,15 +378,17 @@ public class FadeTracker {
 										+ " on " + state.fadeKind.toString()
 										+ " at " + state.getFadePosition());
 							}
-							callbackCallFadePerformed(state.getCallback(),
-									state.fadeKind, state.getFadePosition());
+							callbackCallFadePerformed(compId.component, state
+									.getCallback(), state.fadeKind, state
+									.getFadePosition());
 						}
 						continue;
 					}
 					if (state.isToRepaintParent())
 						parents.add(compId.component.getParent());
-					else
+					else {
 						compId.component.repaint();
+					}
 				}
 			}
 			// if (mapComp.size() == 0)
@@ -429,7 +431,7 @@ public class FadeTracker {
 		}
 	}
 
-	private void callbackCallFadePerformed(
+	private void callbackCallFadePerformed(final Component comp,
 			final FadeTrackerCallback fadeCallback, final FadeKind fadeKind,
 			final float fadePosition) {
 		boolean shouldRunOnEDT = false;
@@ -1146,7 +1148,7 @@ public class FadeTracker {
 	 * @return Fade state of specified component if it's being tracked by
 	 *         <code>this</code> tracker or <code>null</code>.
 	 */
-	public synchronized FadeState getFadeState(Component comp, FadeKind fadeKind) {
+	public FadeState getFadeState(Component comp, FadeKind fadeKind) {
 		return this.getFadeState(comp, null, fadeKind);
 	}
 
@@ -1164,7 +1166,7 @@ public class FadeTracker {
 	 * @return Fade state of specified component if it's being tracked by
 	 *         <code>this</code> tracker or <code>null</code>.
 	 */
-	public synchronized FadeState getFadeState(Component comp, int componentId,
+	public FadeState getFadeState(Component comp, int componentId,
 			FadeKind fadeKind) {
 		return this.getFadeState(comp, new Integer(componentId), fadeKind);
 	}
