@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -79,6 +79,8 @@ public class ComboboxAutoCompletionWidget extends LafWidgetAdapter {
 
 	protected CompletionPlainDocument completionDocument;
 
+	protected Document originalDocument;
+
 	protected ActionMap oldActionMap;
 
 	/**
@@ -277,8 +279,8 @@ public class ComboboxAutoCompletionWidget extends LafWidgetAdapter {
 	 */
 	@Override
 	public void installUI() {
-		final ComboBoxEditor cbe = comboBox.getEditor();
-		final Component cbc = cbe.getEditorComponent();
+		ComboBoxEditor cbe = comboBox.getEditor();
+		Component cbc = cbe.getEditorComponent();
 		if (cbc instanceof JTextComponent) {
 			installTextEditor((JTextComponent) cbc);
 		} else {
@@ -301,6 +303,7 @@ public class ComboboxAutoCompletionWidget extends LafWidgetAdapter {
 		// Code contributed by Thomas Bierhance from
 		// http://www.orbital-computer.de/JComboBox/
 		if (comboBox.isEditable()) {
+			originalDocument = c.getDocument();
 			completionDocument = new CompletionPlainDocument(comboBox);
 			c.setDocument(completionDocument);
 			model = comboBox.getModel();
@@ -417,13 +420,35 @@ public class ComboboxAutoCompletionWidget extends LafWidgetAdapter {
 		changeHandler = null;
 
 		if (comboBox.isEditable()) {
-			uninstallTextEditor(null);
+			ComboBoxEditor cbe = comboBox.getEditor();
+			Component cbc = cbe.getEditorComponent();
+			// uninstallTextEditor(null);
+			if (cbc instanceof JTextComponent)
+				uninstallTextEditor((JTextComponent) cbc);
+			else
+				uninstallTextEditor(null);
 		}
 
 		super.uninstallListeners();
 	}
 
 	protected void uninstallTextEditor(final JTextComponent e) {
+		if ((e != null) && (originalDocument != null)) {
+			if ((completionDocument != null)
+					&& (originalDocument instanceof PlainDocument)) {
+				try {
+					// try updating the contents of the original document
+					String currText = completionDocument.getText(0,
+							completionDocument.getLength());
+					originalDocument.remove(0, originalDocument.getLength());
+					originalDocument.insertString(0, currText, null);
+				} catch (BadLocationException ble) {
+					// fail silently
+				}
+			}
+			// restore the original document
+			e.setDocument(originalDocument);
+		}
 		completionDocument = null;
 		comboBox.removeActionListener(completionActionListener);
 		completionActionListener = null;
@@ -443,13 +468,14 @@ public class ComboboxAutoCompletionWidget extends LafWidgetAdapter {
 			comboBox.setActionMap(oldActionMap);
 			oldActionMap = null;
 		}
+
 	}
 
 	protected void installEditor(final Component c) {
 		if ((c == null) || (editor == c))
 			return;
 
-		final Component last = editor;
+		Component last = editor;
 		if (last != null) {
 			last.removeKeyListener(editorKeyListener);
 			last.removeFocusListener(editorFocusListener);
@@ -481,21 +507,22 @@ public class ComboboxAutoCompletionWidget extends LafWidgetAdapter {
 			String propertyName = e.getPropertyName();
 
 			if (propertyName.equals("editable")) {
-				final boolean oldValue = ((Boolean) e.getOldValue())
-						.booleanValue();
-				final boolean newValue = ((Boolean) e.getNewValue())
-						.booleanValue();
+				boolean oldValue = ((Boolean) e.getOldValue()).booleanValue();
+				boolean newValue = ((Boolean) e.getNewValue()).booleanValue();
+				ComboBoxEditor cbe = comboBox.getEditor();
+				Component cbc = cbe.getEditorComponent();
 				if (!oldValue && newValue
 						&& LafWidgetUtilities.hasAutoCompletion(comboBox)) {
-					final ComboBoxEditor cbe = comboBox.getEditor();
-					final Component cbc = cbe.getEditorComponent();
 					if (cbc instanceof JTextComponent) {
 						installTextEditor((JTextComponent) cbc);
 					} else {
 						installEditor(cbc);
 					}
 				} else if (oldValue && !newValue) {
-					uninstallTextEditor(null);
+					if (cbc instanceof JTextComponent)
+						uninstallTextEditor((JTextComponent) cbc);
+					else
+						uninstallTextEditor(null);
 				}
 			}
 
@@ -503,8 +530,8 @@ public class ComboboxAutoCompletionWidget extends LafWidgetAdapter {
 			// mode on editable comboboxes.
 			if (propertyName.equals(LafWidget.COMBO_BOX_NO_AUTOCOMPLETION)
 					|| propertyName.equals("JComboBox.isTableCellEditor")) {
-				final ComboBoxEditor cbe = comboBox.getEditor();
-				final Component cbc = cbe.getEditorComponent();
+				ComboBoxEditor cbe = comboBox.getEditor();
+				Component cbc = cbe.getEditorComponent();
 				if (LafWidgetUtilities.hasAutoCompletion(comboBox)) {
 					if (cbc instanceof JTextComponent) {
 						installTextEditor((JTextComponent) cbc);
@@ -521,18 +548,16 @@ public class ComboboxAutoCompletionWidget extends LafWidgetAdapter {
 
 			// fix for defect 131 in 2.2_01
 			if (propertyName.equals("editor")) {
-				final ComboBoxEditor oldValue = (ComboBoxEditor) e
-						.getOldValue();
-				final ComboBoxEditor newValue = (ComboBoxEditor) e
-						.getNewValue();
+				ComboBoxEditor oldValue = (ComboBoxEditor) e.getOldValue();
+				ComboBoxEditor newValue = (ComboBoxEditor) e.getNewValue();
 				if ((newValue != null) && (newValue != oldValue)
 						&& LafWidgetUtilities.hasAutoCompletion(comboBox)) {
-					final Component old = (oldValue != null) ? oldValue
+					Component old = (oldValue != null) ? oldValue
 							.getEditorComponent() : null;
 					if (old instanceof JTextComponent) {
 						uninstallTextEditor((JTextComponent) old);
 					}
-					final Component pending = newValue.getEditorComponent();
+					Component pending = newValue.getEditorComponent();
 					if (pending instanceof JTextComponent) {
 						installTextEditor((JTextComponent) pending);
 					} else {
@@ -551,8 +576,8 @@ public class ComboboxAutoCompletionWidget extends LafWidgetAdapter {
 			if (propertyName.equals("model")) {
 				if (LafWidgetUtilities.hasAutoCompletion(comboBox)) {
 					uninstallTextEditor(null);
-					final ComboBoxEditor cbe = comboBox.getEditor();
-					final Component cbc = cbe.getEditorComponent();
+					ComboBoxEditor cbe = comboBox.getEditor();
+					Component cbc = cbe.getEditorComponent();
 					if (cbc instanceof JTextComponent) {
 						installTextEditor((JTextComponent) cbc);
 					} else {
