@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -202,7 +202,8 @@ public class TransitionLayout implements LayoutManager {
 			Component c = parent.getComponent(i);
 			JComponent jc = (JComponent) c;
 			if (!jc.isVisible()
-					|| !Boolean.TRUE.equals(jc.getClientProperty(SHOWING)))
+					|| (hasFades && !Boolean.TRUE.equals(jc
+							.getClientProperty(SHOWING))))
 				setAlpha(jc, new Float(0.0), new Float(0.0), true);
 			if (Boolean.TRUE.equals(jc.getClientProperty(LIMBO)))
 				jc.setVisible(false);
@@ -211,12 +212,16 @@ public class TransitionLayout implements LayoutManager {
 			oldLocations.put(jc, new Rectangle(jc.getBounds()));
 			// jc.putClientProperty(ALPHA, new Float(0.0));
 		}
-		final Map parentOpacityMap = new HashMap();
+		final Map<Component, Boolean> parentOpacityMap = new HashMap<Component, Boolean>();
 		if (hasFades)
 			makeNonOpaque(parent, parentOpacityMap);
 		delegate.layoutContainer(parent);
-		if (hasFades)
-			restoreOpaque(parent, parentOpacityMap);
+		if (hasFades) {
+			restoreOpaque(parent, parentOpacityMap, true);
+			if (parentOpacityMap.size() > 0) {
+				throw new IllegalStateException();
+			}
+		}
 		parent.repaint();
 		// System.out.println("Has " + parent.getComponentCount() + " comps");
 		for (int i = 0; i < parent.getComponentCount(); i++) {
@@ -269,7 +274,7 @@ public class TransitionLayout implements LayoutManager {
 				// jc.putClientProperty(ALPHA, null);
 				if (!oldBounds.equals(newBounds)) {
 					jc.setBounds(oldBounds);
-					final Map opacity = new HashMap();
+					final Map<Component, Boolean> opacity = new HashMap<Component, Boolean>();
 					if (hasFades)
 						makeNonOpaque(jc, opacity);
 					// if (jc instanceof JButton) {
@@ -281,8 +286,12 @@ public class TransitionLayout implements LayoutManager {
 							false, new EDTFadeTrackerAdapter() {
 								@Override
 								public void fadeEnded(FadeKind fadeKind) {
-									if (hasFades)
-										restoreOpaque(jc, opacity);
+									if (hasFades) {
+										restoreOpaque(jc, opacity, true);
+										if (opacity.size() > 0) {
+											throw new IllegalStateException();
+										}
+									}
 									float parentAlpha = getCompositeAlpha(jc
 											.getParent());
 									if (parentAlpha < 1.0f)
@@ -346,7 +355,7 @@ public class TransitionLayout implements LayoutManager {
 				setAlpha(jc, new Float(0.0), new Float(0.0), true);
 				// jc.putClientProperty(ALPHA, new Float(0.0f));
 				jc.setBounds(newBounds);
-				final Map opacity = new HashMap();
+				final Map<Component, Boolean> opacity = new HashMap<Component, Boolean>();
 				makeNonOpaque(jc, opacity);
 				jc.setVisible(true);
 				this.animationStarted();
@@ -361,7 +370,10 @@ public class TransitionLayout implements LayoutManager {
 											new Float(1.0f), true);
 								else
 									clearAlpha(jc, true);
-								restoreOpaque(jc, opacity);
+								restoreOpaque(jc, opacity, true);
+								if (opacity.size() > 0) {
+									throw new IllegalStateException();
+								}
 								animationEnded();
 							}
 
@@ -393,7 +405,7 @@ public class TransitionLayout implements LayoutManager {
 				else
 					setAlpha(jc, new Float(1.0f), new Float(1.0f), true);
 				// jc.putClientProperty(ALPHA, new Float(1.0f));
-				final Map opacity = new HashMap();
+				final Map<Component, Boolean> opacity = new HashMap<Component, Boolean>();
 				makeNonOpaque(jc, opacity);
 				jc.setBounds(oldBounds);
 				jc.putClientProperty(LIMBO, Boolean.TRUE);
@@ -403,7 +415,10 @@ public class TransitionLayout implements LayoutManager {
 						false, new EDTFadeTrackerAdapter() {
 							@Override
 							public void fadeEnded(FadeKind fadeKind) {
-								restoreOpaque(jc, opacity);
+								restoreOpaque(jc, opacity, true);
+								if (opacity.size() > 0) {
+									throw new IllegalStateException();
+								}
 								jc.setBounds(newBounds);
 								jc.setVisible(false);
 								jc.putClientProperty(SHOWING, null);
@@ -461,14 +476,15 @@ public class TransitionLayout implements LayoutManager {
 	 * 
 	 * @param comp
 	 *            Component.
-	 * @param opaquenessSnapshot
+	 * @param opacitySnapshot
 	 *            The "snapshot" map that will contain the original opacity
 	 *            status of the specified component and all its descendants.
 	 */
-	public static void makeNonOpaque(Component comp, Map opaquenessSnapshot) {
+	public static void makeNonOpaque(Component comp,
+			Map<Component, Boolean> opacitySnapshot) {
 		if (comp instanceof JComponent) {
 			JComponent jcomp = (JComponent) comp;
-			opaquenessSnapshot.put(comp, Boolean.valueOf(jcomp.isOpaque()));
+			opacitySnapshot.put(comp, jcomp.isOpaque());
 			jcomp.putClientProperty(ORIGINAL_OPACITY, Boolean.valueOf(jcomp
 					.isOpaque()));
 			// System.out.println(jcomp.getClass().getName() + " : "
@@ -479,7 +495,7 @@ public class TransitionLayout implements LayoutManager {
 		if (comp instanceof Container) {
 			Container cont = (Container) comp;
 			for (int i = 0; i < cont.getComponentCount(); i++)
-				makeNonOpaque(cont.getComponent(i), opaquenessSnapshot);
+				makeNonOpaque(cont.getComponent(i), opacitySnapshot);
 		}
 	}
 
@@ -488,19 +504,20 @@ public class TransitionLayout implements LayoutManager {
 	 * 
 	 * @param comp
 	 *            Component.
-	 * @param opaquenessSnapshot
+	 * @param opacitySnapshot
 	 *            The "snapshot" map that contains the original opacity status
 	 *            of the specified component and all its descendants.
 	 */
-	public static void restoreOpaque(Component comp, Map opaquenessSnapshot) {
+	public static void restoreOpaque(Component comp,
+			Map<Component, Boolean> opacitySnapshot, boolean toCleanup) {
 		if (comp instanceof JComponent) {
 			JComponent jcomp = (JComponent) comp;
 			// snapshot may not contain opacity for table header of a table when
 			// it's used inside tree cell renderer (wrapper in a scroll pane).
-			if (opaquenessSnapshot.containsKey(comp))
-				jcomp.setOpaque(((Boolean) opaquenessSnapshot.get(comp))
-						.booleanValue());
-			else
+			if (opacitySnapshot.containsKey(jcomp)) {
+				jcomp.setOpaque(opacitySnapshot.get(jcomp));
+				opacitySnapshot.remove(jcomp);
+			} else
 				jcomp.setOpaque(true);
 			jcomp.putClientProperty(ORIGINAL_OPACITY, null);
 			// System.out.println(jcomp.getClass().getName() + " --> "
@@ -509,7 +526,19 @@ public class TransitionLayout implements LayoutManager {
 		if (comp instanceof Container) {
 			Container cont = (Container) comp;
 			for (int i = 0; i < cont.getComponentCount(); i++)
-				restoreOpaque(cont.getComponent(i), opaquenessSnapshot);
+				restoreOpaque(cont.getComponent(i), opacitySnapshot, false);
+		}
+
+		if (toCleanup) {
+			for (Map.Entry<Component, Boolean> entry : opacitySnapshot
+					.entrySet()) {
+				if (entry.getKey() instanceof JComponent) {
+					JComponent jcomp = (JComponent) entry.getKey();
+					jcomp.setOpaque(entry.getValue());
+					jcomp.putClientProperty(ORIGINAL_OPACITY, null);
+				}
+			}
+			opacitySnapshot.clear();
 		}
 	}
 
