@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -149,6 +149,7 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 			// add mouse listener to remove the search panel on mouse
 			// click when CTRL button is pressed.
 			this.searchButton.addMouseListener(new MouseAdapter() {
+				@Override
 				public void mousePressed(MouseEvent e) {
 					if ((e.getModifiers() & InputEvent.CTRL_MASK) != 0) {
 						SwingUtilities.invokeLater(new Runnable() {
@@ -299,6 +300,7 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 		 * 
 		 * @see java.awt.Component#setVisible(boolean)
 		 */
+		@Override
 		public void setVisible(boolean aFlag) {
 			super.setVisible(aFlag);
 			if (aFlag)
@@ -406,6 +408,7 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 		 * 
 		 * @see java.lang.Object#toString()
 		 */
+		@Override
 		public String toString() {
 			StringBuffer sb = new StringBuffer();
 			if (this.menuElements != null) {
@@ -615,6 +618,7 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 		});
 	}
 
+	@Override
 	public void setComponent(JComponent jcomp) {
 		super.setComponent(jcomp);
 		this.jmb = (JMenuBar) jcomp;
@@ -625,6 +629,7 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 	 * 
 	 * @see org.jvnet.lafwidget.LafWidgetAdapter#installUI()
 	 */
+	@Override
 	public void installUI() {
 		final LafWidgetSupport lafSupport = LafWidgetRepository.getRepository()
 				.getLafSupport();
@@ -640,6 +645,7 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 		// JMenu one entry before the last (so that our search panel
 		// will always be the last).
 		this.jmb.addContainerListener(new ContainerAdapter() {
+			@Override
 			public void componentAdded(ContainerEvent e) {
 				if (!(e.getChild() instanceof JMenu))
 					return;
@@ -683,6 +689,7 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 	 * 
 	 * @see org.jvnet.lafwidget.LafWidgetAdapter#uninstallUI()
 	 */
+	@Override
 	public void uninstallUI() {
 		this.jmb.remove(this.searchPanel);
 		super.uninstallUI();
@@ -693,6 +700,7 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 	 * 
 	 * @see org.jvnet.lafwidget.LafWidgetAdapter#installListeners()
 	 */
+	@Override
 	public void installListeners() {
 		super.installListeners();
 
@@ -730,6 +738,7 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 	 * 
 	 * @see org.jvnet.lafwidget.LafWidgetAdapter#uninstallListeners()
 	 */
+	@Override
 	public void uninstallListeners() {
 		this.jmb.removePropertyChangeListener(this.propertyListener);
 		this.propertyListener = null;
@@ -864,6 +873,8 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 							.getPreferredSize().width;
 					int tbHeight = this.searchPanel.searchStringField
 							.getPreferredSize().height;
+					// make the text field fit in the available height
+					tbHeight = Math.min(tbHeight, height - 2);
 					this.searchPanel.searchStringField.setBounds(x,
 							(height - tbHeight) / 2, tbWidth, tbHeight);
 
@@ -896,6 +907,8 @@ public class MenuSearchWidget extends LafWidgetAdapter implements Resettable {
 							.getPreferredSize().width;
 					int tbHeight = this.searchPanel.searchStringField
 							.getPreferredSize().height;
+					// make the text field fit in the available height
+					tbHeight = Math.min(tbHeight, height - 2);
 					this.searchPanel.searchStringField.setBounds(x - tbWidth,
 							(height - tbHeight) / 2, tbWidth, tbHeight);
 
