@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -328,7 +328,7 @@ public class TabOverviewDialog extends JDialog {
 					// .getColor("Label.foreground"));
 
 					// System.err.println("Setting image on " + tabIndex);
-					previewControls[tabIndex].setImage(result);
+					previewControls[tabIndex].setRichImage(result);
 					// System.err.println("Set image on " + tabIndex);
 					previewControls[tabIndex].repaint();
 					// previewControls.add(caroselComponent);
@@ -503,8 +503,7 @@ public class TabOverviewDialog extends JDialog {
 						g2d.setColor(UIManager.getColor("Label.background"));
 						g2d.fillRect(0, 0, pWidth, pHeight);
 						ReflectedImageLabel ril = (ReflectedImageLabel) caroselMenu
-								.add(placeHolder, tabPane.getTitleAt(i),
-										pWidth, pHeight);
+								.add(placeHolder, tabPane.getTitleAt(i));
 						ril.setForeground(UIManager
 								.getColor("Label.foreground"));
 						ril.setBackground(UIManager
@@ -568,7 +567,7 @@ public class TabOverviewDialog extends JDialog {
 					// .getColor("Label.foreground"));
 
 					// System.err.println("Setting image on " + tabIndex);
-					previewControls[tabIndex].setImage(result);
+					previewControls[tabIndex].setRichImage(result);
 					// System.err.println("Set image on " + tabIndex);
 					previewControls[tabIndex].repaint();
 					// previewControls.add(caroselComponent);
@@ -1169,7 +1168,7 @@ public class TabOverviewDialog extends JDialog {
 	 * @return Tab overview dialog for the specified tabbed pane.
 	 */
 	public static TabOverviewDialog getOverviewDialog(JTabbedPane tabPane) {
-		TabPreviewPainter previewPainter = LafWidgetUtilities2
+		final TabPreviewPainter previewPainter = LafWidgetUtilities2
 				.getTabPreviewPainter(tabPane);
 		String title = previewPainter.toUpdatePeriodically(tabPane) ? MessageFormat
 				.format(LafWidgetUtilities.getResourceBundle(tabPane)
@@ -1183,7 +1182,7 @@ public class TabOverviewDialog extends JDialog {
 		Rectangle dialogScreenBounds = previewPainter
 				.getPreviewDialogScreenBounds(tabPane);
 		TabOverviewKind overviewKind = previewPainter.getOverviewKind(tabPane);
-		TabOverviewDialog overviewDialog = new TabOverviewDialog(tabPane,
+		final TabOverviewDialog overviewDialog = new TabOverviewDialog(tabPane,
 				overviewKind, frameForModality, isModal,
 				dialogScreenBounds.width, dialogScreenBounds.height);
 		overviewDialog.setTitle(title);
@@ -1192,6 +1191,41 @@ public class TabOverviewDialog extends JDialog {
 		overviewDialog.setSize(dialogScreenBounds.width,
 				dialogScreenBounds.height);
 
+		// make sure that the tab overview dialog is disposed when
+		// it loses focus
+		final PropertyChangeListener activeWindowListener = new PropertyChangeListener() {
+			public void propertyChange(PropertyChangeEvent evt) {
+				if ("activeWindow".equals(evt.getPropertyName())) {
+					if (overviewDialog == evt.getOldValue()) {
+						if (previewPainter.toDisposeOverviewOnFocusLoss()) {
+							overviewDialog.dispose();
+						}
+					}
+				}
+			}
+		};
+		KeyboardFocusManager.getCurrentKeyboardFocusManager()
+				.addPropertyChangeListener(activeWindowListener);
+
+		// make sure that when the window with the tabbed pane is
+		// closed, the tab overview dialog is disposed.
+		final Window tabWindow = SwingUtilities.getWindowAncestor(tabPane);
+		final WindowListener tabWindowListener = new WindowAdapter() {
+			@Override
+			public void windowClosed(WindowEvent e) {
+				overviewDialog.dispose();
+			}
+		};
+		tabWindow.addWindowListener(tabWindowListener);
+		overviewDialog.addWindowListener(new WindowAdapter() {
+			@Override
+			public void windowClosed(WindowEvent e) {
+				tabWindow.removeWindowListener(tabWindowListener);
+				KeyboardFocusManager.getCurrentKeyboardFocusManager()
+						.removePropertyChangeListener(activeWindowListener);
+			}
+		});
+
 		return overviewDialog;
 	}
 }
